"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.timedFetch = void 0;
const core_interface_1 = require("@ideal-postcodes/core-interface");
const { IdealPostcodesError } = core_interface_1.errors;
/**
 * Returns Ideal Postcode timeout error
 *
 * @hidden
 */
const timeoutError = (timeout, request, url) => new IdealPostcodesError({
    message: `Request timed out after ${timeout}ms`,
    httpStatus: 0,
    metadata: { request, url },
});
/**
 * Wraps native fetch with a promise that can timeout
 *
 * Executes AbortController if present on timed out request
 *
 * @hidden
 */
const timedFetch = (url, requestInit, timeout, abortController) => new Promise((resolve, reject) => {
    setTimeout(() => {
        if (abortController)
            abortController.abort();
        reject(timeoutError(timeout, requestInit, url));
    }, timeout);
    fetch(url, requestInit).then(resolve, reject);
});
exports.timedFetch = timedFetch;
